//------------------------------------------------------------------
#property copyright   "mladen"
#property link        "www.forex-tsd.com"
#property version     "1.00"
//------------------------------------------------------------------
#property indicator_separate_window
#property indicator_buffers 4
#property strict

//
//
//
//
//

enum enCorrType
{
   cor_spe,  // Spearman rank correlation
   cor_pea   // Pearson rank correlation
};

extern int            CorPeriod         = 32;                 // Correlation rank
extern enCorrType     Type              = cor_pea;            // Correlation type
extern int            CalcBars          = 0;                  // Bars to calculate
extern color          colorBarDn        = clrRed;             // Color for down bar
extern color          colorBarUp        = clrLimeGreen;       // Color for up bar
extern color          colorBarNeutral   = clrDimGray;         // Color for neutral bar
extern color          colorWickUp       = clrLimeGreen;       // Color for up wick
extern color          colorWickDn       = clrRed;             // Color for down wick
extern color          colorWickNeutral  = clrDimGray;         // Color for neutral wick
extern int            widthWick         = 1;                  // Wick width
extern int            widthBody         = 3;                  // Body width
extern string         UniqueID          = "corr candles 1";   // Unique ID

double colors[];
double open[];
double close[];
double high[];
double low[];
int    window;
string shortName;

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int init()
{
   IndicatorBuffers(4);
   SetIndexBuffer(0, open);      SetIndexStyle(0,DRAW_LINE,EMPTY,EMPTY,clrNONE);
   SetIndexBuffer(1, high);      SetIndexStyle(1,DRAW_LINE,EMPTY,EMPTY,clrNONE);
   SetIndexBuffer(2, low);       SetIndexStyle(2,DRAW_LINE,EMPTY,EMPTY,clrNONE);
   SetIndexBuffer(3, close);     SetIndexStyle(3,DRAW_LINE,EMPTY,EMPTY,clrNONE);

   string add = (Type==cor_pea) ? " Pearson " : " Spearman ";
   shortName = UniqueID+": "+add+" candles ("+(string)CorPeriod+")";
   IndicatorShortName(shortName);
return(0);
}
int deinit()
{
   string lookFor       = UniqueID+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i); if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
   return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
           int bars  = (CalcBars>0) ? CalcBars : Bars; 
           int limit = fmin(bars-counted_bars,bars-1); 
           window = WindowFind(shortName);
   
   //
   //
   //
   //
   //
   
   for (int i=limit; i>=0; i--) 
   {
      if (Type==cor_spe)
      {
        open[i]  = iSpearman(iMA(NULL,0,1,0,MODE_SMA,PRICE_OPEN,i), CorPeriod,i,bars,0);
        close[i] = iSpearman(iMA(NULL,0,1,0,MODE_SMA,PRICE_CLOSE,i),CorPeriod,i,bars,1);
        high[i]  = iSpearman(iMA(NULL,0,1,0,MODE_SMA,PRICE_HIGH,i), CorPeriod,i,bars,2);
        low[i]   = iSpearman(iMA(NULL,0,1,0,MODE_SMA,PRICE_LOW,i),  CorPeriod,i,bars,3);
      }
      else
      {
        open[i]  = iPearson(iMA(NULL,0,1,0,MODE_SMA,PRICE_OPEN,i), CorPeriod,i,bars,0);
        close[i] = iPearson(iMA(NULL,0,1,0,MODE_SMA,PRICE_CLOSE,i),CorPeriod,i,bars,1);
        high[i]  = iPearson(iMA(NULL,0,1,0,MODE_SMA,PRICE_HIGH,i), CorPeriod,i,bars,2);
        low[i]   = iPearson(iMA(NULL,0,1,0,MODE_SMA,PRICE_LOW,i),  CorPeriod,i,bars,3);
      }
      
      //
      //
      //
      //
      //
    
      color  theBarColor  = colorBarNeutral;
      color  theWickColor = colorWickNeutral;
      if (close[i]<open[i]) { theBarColor = colorBarDn; theWickColor = colorWickDn; }
      if (close[i]>open[i]) { theBarColor = colorBarUp; theWickColor = colorWickUp; }
               
      drawBar((int)Time[i],high[i],low[i],open[i],close[i],theBarColor,theWickColor);
   }
return(0);
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

#define pearsonInstances 4
double workPearson[][pearsonInstances];
double iPearson(double value, int period, int i, int bars, int instanceNo=0)
{
   if (ArrayRange(workPearson,0)!=bars) ArrayResize(workPearson,bars); i=bars-i-1; workPearson[i][instanceNo]=value;
   
   //
   //
   //
   //
   //
   
   double SumXY=0; double SumXX=0; double SumYY=0; double SumY=0; double SumX=0;         
   for(int k=0; k<period && (i-k)>=0; k++)
   {
      double val = workPearson[i-k][instanceNo];
             SumX  += val;  
             SumY  += k;  
             SumXX += val*val;
             SumYY += k*k;         
             SumXY += val*k; 
   }
   double SXY    =  period*SumXY-SumY*SumX; 
   double SXXYY  = (period*SumXX-SumX*SumX)*(period*SumYY-SumY*SumY);
   double result = (SXXYY!=0) ? -SXY/(MathSqrt(MathAbs(SXXYY))) : 0;
   return(result);
}

//
//
//
//
//

#define spearmanInstances 4
double workSpearman[][spearmanInstances];
double iSpearman(double value, int period, int i, int bars, int instanceNo=0)
{
   if (ArrayRange(workSpearman,0)!=bars) ArrayResize(workSpearman,bars); i=bars-i-1; workSpearman[i][instanceNo]=value;

   //
   //
   //
   //
   //
      
   double total=0; 
   double data[]; ArrayResize(data, period); ArrayInitialize(data,0);
      for (int k=0; k<period && (i-k)>=0; k++) data[k] = workSpearman[i-k][instanceNo];
      for (int k=0; k<period; k++) { int max = ArrayMaximum(data); total += (max-k)*(max-k); data[max] = 0; }
	return(1.0-6.0*total/(period*(period*period-1.0)));
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void drawBar(int bTime, double prHigh, double prLow, double prOpen, double prClose, color barColor, color wickColor)
{
   string oName;
          oName = shortName+":"+(string)bTime+"w";
            if (ObjectFind(oName) < 0) ObjectCreate(oName,OBJ_TREND,window,bTime,0,bTime,0);
                 ObjectSet(oName, OBJPROP_PRICE1,prHigh);
                 ObjectSet(oName, OBJPROP_PRICE2,prLow);
                 ObjectSet(oName, OBJPROP_COLOR, wickColor);
                 ObjectSet(oName, OBJPROP_WIDTH, widthWick);
                 ObjectSet(oName, OBJPROP_RAY, false);
                 ObjectSet(oName, OBJPROP_BACK, true);
           
         oName = shortName+":"+(string)bTime+"b";
            if (ObjectFind(oName) < 0) ObjectCreate(oName,OBJ_TREND,window,bTime,0,bTime,0);
                 ObjectSet(oName, OBJPROP_PRICE1,prOpen);
                 ObjectSet(oName, OBJPROP_PRICE2,prClose);
                 ObjectSet(oName, OBJPROP_COLOR, barColor);
                 ObjectSet(oName, OBJPROP_WIDTH, widthBody);
                 ObjectSet(oName, OBJPROP_RAY, false);
                 ObjectSet(oName, OBJPROP_BACK, true);
}